const pdf = require('pdf-extraction')
const fs = require('fs')

const Mode = {
  Start: 'start',
  Question: 'question',
  Categories: 'categories',
  Clues: 'clues',
}

const extractPuzzle = async (pdfPath) => {
  let dataBuffer = fs.readFileSync(pdfPath)
  return pdf(dataBuffer).then((data) => {
    let lines = data.text.split('\n')
                    .filter(line => line.length > 0)
                    .filter(line => !line.includes('brainzilla.com'))
                    .map(line => line.replace(/ /g, ' ')) // Note: This is a nbsp -> space replacement

    let puzzle = {title: null, question: '', categories: {}, clues: []}
    let mode = Mode.Start
    let currentEntry = ''
    for(let i = 0; i < lines.length; ++i) {
      let line = lines[i]
      switch(mode) {
        case Mode.Start: {
          puzzle.title = line
          mode = Mode.Question
          break;
        }
        case Mode.Question: {
          if(line.includes('#1')) {
            mode = Mode.Categories
          } else {
            puzzle.question += line
          }
          break;
        }
        case Mode.Categories: {
          if(line.startsWith('­') || line.startsWith('-')) {
            if(currentEntry.length > 0) {
              if(currentEntry.includes(', $')) { // Note: Silly workaround to deal with commas in pricing
                currentEntry = currentEntry.replace(/, \$/g, '^^^').replace(/,/g, '_').replace(/\^\^\^/g, ',$')
              }
              let [category, ...variants] = currentEntry.split(/[:,]/)
              if(variants.length > 0) {
                if(variants.length === 10) {
                  let v = []
                  for(let j = 0; j < variants.length; j+=2) {
                    v.push(`${variants[j]}_${variants[j+1]}`)
                  }
                  variants = v
                }
                puzzle.categories[category] = variants.map(v => v.trim().replace(/[­-]/g, '_'))
                currentEntry = ''
              }
            }
            mode = Mode.Clues
            --i
          } else {
            if(line.includes(':')) {
              if((currentEntry.includes('AM') || currentEntry.includes('PM')) &&
                 (line.includes('AM') || line.includes('PM'))) {
                currentEntry += line
                break;
              } else if(currentEntry.length > 0) {
                if(currentEntry.includes(', $')) { // Note: Silly workaround to deal with commas in pricing
                  currentEntry = currentEntry.replace(/, \$/g, '^^^').replace(/,/g, '_').replace(/\^\^\^/g, ',$')
                }
                let [category, ...variants] = currentEntry.split(/[:,]/)
                if(variants.length > 0) {
                  if(variants.length === 10) {
                    let v = []
                    for(let j = 0; j < variants.length; j+=2) {
                      v.push(`${variants[j]}_${variants[j+1]}`)
                    }
                    variants = v
                  }
                  puzzle.categories[category] = variants.map(v => v.trim().replace(/[­-]/g, '_'))
                }
              }
              currentEntry = line
            } else {
              currentEntry += line
            }
          }
          break;
        }
        case Mode.Clues: {
          if(line.startsWith('­') || line.startsWith('-')) {
            if(currentEntry.length > 2) {
              puzzle.clues.push(currentEntry.replace(/[­-]/g, '_'))
            }
            currentEntry = line.slice(2)
          } else {
            currentEntry += line
          }
          break;
        }
      }
    }
    if(currentEntry.length > 0) puzzle.clues.push(currentEntry.replace(/­/g, '_'))
    puzzle.clues = puzzle.clues.map(clue => clue.replace(/\t/g, ' '))
    return puzzle
  })
}

module.exports = extractPuzzle
