const puzzles = (() => {
  const fs = require('fs')
  let paths = fs.readdirSync('puzzles')

  return paths.map(p => {
    return {
      title: p.split('.')[0].replace(/-/g, ' '),
      searchName: p.split('.')[0],
      path: `puzzles/${p}`,
    }
  })
})();

const solve = require('./solver')
const extract = require('./pdf')
const { parse, tag, analyze } = require('./parse')
const util = require('util')

const repl = () => {
  const completer = (line) => {
    let firstSpace = line.indexOf(' ')
    if(firstSpace < 0) {
      let commands = 'solve extract list tag parse analyze quit'.split(' ')
      let hits = commands.filter(c => c.startsWith(line))
      return [hits.length ? hits : commands, line]
    }
    let needle = line.slice(firstSpace + 1)
    let hits = puzzles.filter((p) => p.searchName.startsWith(needle)).map(p => p.searchName)
    return [hits.length ? hits : puzzles.map(p => p.searchName), needle]
  }
  const rl = require('readline').createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: '> ',
    completer: completer,
  })

  rl.on('SIGINT', () => {
    rl.write('Exiting...\n')
    rl.close()
  })

  rl.prompt()

  rl.on('line', async (line) => {
    let [command, ...args] = line.trim().split(' ')
    switch (command) {
      case 'extract': {
        for(let i=0; i < args.length; ++i) {
          console.log(await extract(`puzzles/${args[i]}.pdf`))
        }
        break
      }
      case 'ls':
      case 'list': {
        console.log(puzzles.map(p => p.searchName))
        break
      }
      case 'solve': {
        for(let i=0; i < args.length; ++i) {
          console.log(await solve(`puzzles/${args[i]}.pdf`))
        }
        break
      }
      case 'tag': {
        for(let i=0; i < args.length; ++i) {
          console.log(util.inspect(tag(await extract(`puzzles/${args[i]}.pdf`)), {depth: 5}))
        }
        break
      }
      case 'parse': {
        for(let i=0; i < args.length; ++i) {
          console.log(util.inspect(parse(tag(await extract(`puzzles/${args[i]}.pdf`))), { depth: null}))
        }
        break
      }
      case 'analyze': {
        for(let i=0; i < args.length; ++i) {
          console.log(util.inspect(analyze(parse(tag(await extract(`puzzles/${args[i]}.pdf`)))), { depth: null}))
        }
        break
      }
      case 'exit':
      case 'quit': {
        rl.close()
        return
      }
      default: {
        console.log(`unknown command: '${command}'`)
        break
      }
    }
    rl.prompt()
  })
}

module.exports = repl
